# $Id: alert.bro,v 1.3 2004/09/02 15:58:38 vern Exp $

type Alert: enum {
	Unspecified,	# place-holder - shouldn't be used
};

type alert_info: record {
	alert: Alert;
	msg: string;
	sub: string &optional;	# sub-message

	conn: connection &optional;	# connection associated with alert
	iconn: icmp_conn &optional;	# associated ICMP "connection"
	id: conn_id &optional;	# connection-ID, if we don't have a connection handy
	src: addr &optional;	# source address, if we don't have a connection
	dst: addr &optional;	# destination address

	p: port &optional;	# associated port, if we don't have a conn.

	# The following are detailed attributes that are associated with some
	# alerts, but not all.

	user: string &optional;

	filename: string &optional;

	method: string &optional;
	URL: string &optional;

	n: count &optional;	# associated count, or perhaps status code

	# Source that raised the last event before the alert was generated.
	# Automatically set by ALERT().
	event_src: event_source &optional;
};

type AlertAction: enum {
	# Similar to WeirdAction in weird.bro.
	ALERT_IGNORE, ALERT_FILE,
	ALERT_LOG_ALWAYS, ALERT_LOG_PER_CONN,
	ALERT_LOG_PER_ORIG, ALERT_LOG_ONCE,
};

type alert_policy_item: record {
	result: AlertAction;
	pred: function(a: alert_info): bool;
	priority: count;
};

global alert_policy: set[alert_policy_item] = {
	[$pred(a: alert_info) = { return T; },
	 $result = ALERT_LOG_ALWAYS,
	 $priority = 0],
} &redef;

# table that maps alerts into a function that should be called
# to determine the action.
global alert_action_filters:
	table[Alert] of function(a: alert_info): AlertAction &redef;

function ignore_alert(a: alert_info): AlertAction
	{
	return ALERT_IGNORE;
	}

function file_alert(a: alert_info): AlertAction
	{
	return ALERT_FILE;
	}

global alert_tallies: table[string] of count &default = 0;

function tally_alert(s: string)
	{
	++alert_tallies[s];
	}

function tally_alert_type(a: alert_info): AlertAction
	{
	tally_alert(fmt("%s", a$alert));
	return ALERT_FILE;
	}

event bro_done()
	{
	for ( s in alert_tallies )
		{
		local n = alert_tallies[s];
		log fmt("%s (%d time%s)", s, n, n > 1 ? "s" : "");
		}
	}

global alert_file = open_log_file("alert") &redef;

# This handler is useful for processing alerts after the alert filters
# have been applied and yielded an AlertAction.
#
# It's tempting to make the default handler do the logging and
# printing to alert_file, rather than ALERT.  I hesitate to do that,
# though, because it perhaps could slow down notification, because
# in the absence of event priorities, the event would have to wait
# behind any other already-queued events.
event alert_action(a: alert_info, action: AlertAction)
	{
	}

# Given a string, returns an escaped version suitable for being
# printed in the colon-separated alert format.  This means that
# (1) any colons are escaped using '\', and (2) any '\'s are
# likewise escaped.
function alert_escape(s: string): string
	{
	s = subst_string(s, "\\", "\\\\");
	return subst_string(s, ":", "\\:");
	}

function ALERT(a: alert_info)
	{
	# Fill in some defaults.
	if ( ! a?$id && a?$conn )
		a$id = a$conn$id;

	if ( ! a?$src && a?$id )
		a$src = a$id$orig_h;
	if ( ! a?$dst && a?$id )
		a$dst = a$id$resp_h;

	if ( ! a?$p && a?$id )
		a$p = a$id$resp_p;

	if ( ! a?$src && a?$iconn )
		a$src = a$iconn$orig_h;
	if ( ! a?$dst && a?$iconn )
		a$dst = a$iconn$resp_h;

	# Event source may already be set if we got this
	# alert from a remote peer.
	if ( ! a?$event_src )
		a$event_src = get_event_source();

	local action = match a using alert_policy;

	if ( a$alert in alert_action_filters )
		action = alert_action_filters[a$alert](a);

	if ( action != ALERT_IGNORE && action != ALERT_FILE )
		{
		local descr = "";
		if ( is_remote_event() )
			{
			if ( a$event_src$descr != "" )
				descr = fmt("<%s> ", a$event_src$descr );
			else
				descr = fmt("<%s/%s> ", a$event_src$host,
							a$event_src$p);
			}

		log fmt("%s %s%s", a$alert, descr, a$msg);
		}

	if ( action != ALERT_IGNORE )
		{
		local t = fmt("%.06f", network_time());
		local alert = fmt("%s", a$alert);
		local alert_act = fmt("%s", action);
		local event_src =
			is_remote_event() ?
				(a$event_src$descr != "" ?
					a$event_src$descr :
					fmt("%s/%s", a$event_src$host,
						a$event_src$p)) : "";
		local src_addr = a?$src ? fmt("%s", a$src) : "";
		local dst_addr = a?$dst ? fmt("%s", a$dst) : "";

		local src_port = (a?$id && a$id$orig_h == a$src) ?
					fmt("%s", a$id$orig_p) : "";
		local dst_port = (a?$id && a$id$resp_h == a$dst) ?
					fmt("%s", a$id$resp_p) : "";

		local user = a?$user ? alert_escape(a$user) : "";
		local filename = a?$filename ? alert_escape(a$filename) : "";
		local method = a?$method ? alert_escape(a$method) : "";
		local URL = a?$URL ? alert_escape(a$URL) : "";
		local n = a?$n ? fmt("%s", a$n) : "";
		local msg = a?$msg ? alert_escape(a$msg) : "";
		local sub_msg = a?$sub ? alert_escape(a$sub) : "";

		local info =
			fmt("%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s",
				t, alert, alert_act, event_src,
				src_addr, src_port, dst_addr, dst_port,
				user, filename, method, URL, n, msg, sub_msg);

		print alert_file, info;
		}

@ifdef ( IDMEF_support )
	if ( a?$id )
		generate_idmef(a$id$orig_h, a$id$orig_p,
			       a$id$resp_h, a$id$resp_p);
@endif

	event alert_action(a, action);
	}
